<?php
/**
 * Created by PhpStorm.
 * User: Denis
 * Date: 26.03.2015
 * Time: 15:35
 */

//namespace cc_ads_manage;

class cc_DBUpdater {
    static $currVersion;// = '1.0.0'; 




    static function version_to_func_name($newVersion){
        $tempNew = str_replace(".", "_", $newVersion);

        return "update_to_$tempNew";
    }

    private static function get_outstanding_functions( $oldVersion ){
        $functions = get_class_methods(UAM_FunctionContainer::class); 
        $funcName = self::version_to_func_name($oldVersion); /* get the function which was (potentially) executed last time */

        $filterFunctions = function($v) {
        return strpos($v, 'update_to_') === 0;
        }; 

        $update_funcs = array_filter($functions, $filterFunctions); /* we want only update_to_ functions! */
        $update_funcs[] = $funcName; /* //insert last executed function name to make sure it is in the array */
        $update_funcs = array_unique($update_funcs); /* //remove our entry if it was already contained */
        natsort($update_funcs); /*  //sort the functions s.t. they are executed in the correct order! */
        $update_funcs = array_values($update_funcs); /* //natsort sorts the keys too. we do not want this, cut them off. */

        $oldVersionIndex = array_search($funcName, $update_funcs); /* //now lookup the current index, s.t. we can apply all remaining functions */

        $outstanding_funcs = array_slice($update_funcs, $oldVersionIndex + 1);
        return $outstanding_funcs;
  } 

    static function updateDB(){
        global $cc_uam_config;
        $varTemp = get_plugin_data( dirname(__FILE__).'/ultimate-ads-manager.php', false, false );
        self::$currVersion = $varTemp['Version'];
        $oldVersion = get_option( 'cc_ads_manage_curr_version', '1.0.0' ); 


        self::register(); //try to register

        $funcContainer = new UAM_FunctionContainer();
        $funcContainer->legacy();

        if($oldVersion === self::$currVersion)return;

        //flush_rewrite_rules();


        update_option( 'cc_ads_manage_curr_version',self::$currVersion );

        $outstanding_funcs = self::get_outstanding_functions($oldVersion); 

        update_option('codeneric_uam_register_status', 500); //data is outdated, update profile



        for($i = 0; $i< count($outstanding_funcs); $i++ ){
            $funcName = $outstanding_funcs[$i];  

            if(method_exists ($funcContainer, $funcName)){ //we have to perform some operations on the database
                $funcContainer->$funcName();
            }
        } 

        /////////// AFTER INSTALL/UPGRADE
        do_action($cc_uam_config['p'].'/base-plugin-updated');

//        require_once plugin_dir_path( __FILE__ ) . 'config.php';





    }

    static function update_to_premium(){
        global $cc_uam_config;
        $uuid = get_option( 'codeneric_uam_uuid' );

//        $status = array('text' => 'Fetching premium plugin...', 'code' => 0);


        $prem_download_url = $cc_uam_config['wpps_url']."/paid?plugin_id=$uuid";
        $res = wp_remote_get($prem_download_url);
        if(isset($res) && is_array($res) && isset($res['response']) && isset($res['response']['code'])){
            $code = $res['response']['code'];
            if($code === 402){
                return array('text' => 'Payment required.', 'code' => 402);
            }elseif($code !== 200){
                return array('text' => 'Something went wrong, try again later.', 'code' => 503);
            }
        }else{
            return array('text' => 'Codeneric Server currently down, try again later.', 'code' => 0);
        }

        $prem_download_url = $cc_uam_config['wpps_url']."/premium/uam/".cc_DBUpdater::$currVersion."?plugin_id=$uuid";

        if ( class_exists( 'ZipArchive' ) ) {
            $path = download_url($prem_download_url);
            if(is_string($path)){
                $zip_path = plugin_dir_path( __FILE__ ) . 'premium.zip';
                $zip_dest = plugin_dir_path( __FILE__ );
                rename($path, $zip_path);

                cc_DBUpdater::rrmdir(plugin_dir_path( __FILE__ ) . 'premium');

                $zip = new ZipArchive();
                if ($zip->open($zip_path) === true &&  $zip->extractTo($zip_dest) === true) {
                    $zip->close();
                    unlink($zip_path);
                    return true;
                }else{

                    return array('text' => 'Error occurred during upgrade.', 'code' => 42);
                }
            }

        }else{
            return array('text' => 'Zip-Module not found, upgrade to premium only manually possible.', 'code' => 42);
        }

    }


    static function register(){
        global $cc_uam_config;
        require_once (dirname(__FILE__).'/includes/common.php');

        $reg_status = get_option('codeneric_uam_register_status');
        $reg_status = $reg_status !== false ? intval($reg_status) : 500;

        //print_r($config);
        if($reg_status !== 200){
            $blog_url = get_bloginfo( 'url' );
            $blog_version = get_bloginfo( 'version' );
            $blog_lang = get_bloginfo( 'language' );
            $blog_id = get_option('codeneric_uam_uuid');
            if($blog_id === false)
                update_option('codeneric_uam_uuid', Ultimate_Ads_Manager_Base_Common::gen_uuid() );
            $p_v = cc_DBUpdater::$currVersion;
            $res = wp_remote_get( $cc_uam_config['wpps_url'] . "/register/?plugin_id=$blog_id&wp_version=$blog_version&wp_url=$blog_url&wp_lang=$blog_lang&product=uam&plugin_version=$p_v" );
            $code = 500;

            if(isset($res) && is_array($res) && isset($res['response']) && isset($res['response']['code'])){
                $code = $res['response']['code'];
            }


            update_option('codeneric_uam_register_status', $code);
        }
    }

    static function rrmdir( $dir ) {
        if ( is_dir( $dir ) ) {
            $objects = scandir( $dir );
            foreach ( $objects as $object ) {
                if ( $object != "." && $object != ".." ) {
                    if ( filetype( $dir . "/" . $object ) == "dir" ) {
                        cc_DBUpdater::rrmdir( $dir . "/" . $object );
                    } else {
                        unlink( $dir . "/" . $object );
                    }
                }
            }
            reset( $objects );
            return rmdir( $dir );
        }
    }


}

class UAM_FunctionContainer{
    

    
    function update_to_1_2_1(){
        global $cc_uam_config;
        $args        = array(
            'posts_per_page' => - 1,
            'orderby'        => 'date',
            'order'          => 'DESC',
            'post_type'      => $cc_uam_config['custom_post_slug'],
            'post_status'    => 'publish'
            //'suppress_filters' => true
        );
        $posts_array = get_posts( $args );
        foreach($posts_array as $p ){
            $meta = get_post_meta( $p->ID, 'ad', true );
            if(isset($meta) && isset($meta['images']) && isset($meta['referral_url'])){ //old ad, convert it!
                if(is_array($meta['images']) && count($meta['images']) > 0 ){
                    $img_id = array_keys($meta['images']);
                    $img_id = $img_id[0];
                    $vals = array_values($meta['images']);
                    if(isset($vals[0]['url']) && isset($vals[0]['title'])){
                        if(is_string($vals[0]['url']) && is_string($vals[0]['url']) && is_string($meta['referral_url'])){
                            $new_meta = array(
                                'image_ad_referral_url' => $meta['referral_url'],
                                'image_ad_title'        => $vals[0]['title'],
                                'image_ad_uri'          => $vals[0]['url'],
                                'image_ad_id'           => $img_id,
                                'type'                  => 'image_ad'
                                );
                            update_post_meta($p->ID, 'ad', $new_meta);

                        }
                    }
                }
            }
        }
    }

    function update_to_2_0_0(){
        global $cc_uam_config;

        require_once dirname(__FILE__).'/includes/util.php';
        $args        = array(
            'posts_per_page' => -1,
            'orderby'        => 'date',
            'order'          => 'DESC',
            'post_type'      => $cc_uam_config['custom_post_slug']
//            'post_status'    => 'publish'
            //'suppress_filters' => true
        );
        $posts_array = get_posts( $args );
        foreach ($posts_array as $post){
            $ad = get_post_meta($post->ID, 'ad', true);
            if(is_array($ad) && isset($ad['type'])){
                if($ad['type'] === 'image_ad'){
                    $image_id = $ad['image_ad_id'];
                    $src = wp_get_attachment_image_src( $image_id, 'full', false );
                    $width = $src[1];
                    $height = $src[2];
                    $form = Ultimate_Ads_Manager_Util::nearest_form($width,$height);
                    $ad['forms'] = array($form);
                }else{
                    $width = 1;
                    $height = 1;
                    $form = Ultimate_Ads_Manager_Util::nearest_form($width,$height);
                    $ad['forms'] = array($form);
                }
                update_post_meta($post->ID, 'ad', $ad);
            }
        }
    }


    function update_to_2_0_2(){
        global $cc_uam_config;
        /////////////// Database Stuff ////////////////////////
        global $wpdb;

        $table_name = $cc_uam_config['table_name_events'];
        /*
                if(UAM_Config::$ENV === 'development'){
                    $wpdb->query( " DROP TABLE $table_name"	);
                }*/

        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
		  id   bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
		  time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		  type tinyint NOT NULL,
		  uuid bigint(20) UNSIGNED NOT NULL,
		  ip   tinytext DEFAULT '' NOT NULL,
		  ad_id   bigint(20) UNSIGNED NOT NULL,
		  ad_slide_id   tinyint UNSIGNED NOT NULL,
		  place_id   bigint(20) UNSIGNED NOT NULL,
		  UNIQUE KEY id (id)
		) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );


    }



    function update_to_2_2_2(){
        //NOTE: although this update should have happened in version 2.1.0, it was not executed automatically
        //      because the update process was too brittle (multiple sources of truth)!
        //      Thus, we changed this methods name from update_to_2_1_0 to update_to_2_2_2, such that it gets executed on the next update
        //      (updating from 2.2.1 to 2.2.2)!
        global $cc_uam_config;
        /////////////// Database Stuff ////////////////////////
        global $wpdb;

        $table_name = $cc_uam_config['table_name_events'];
        /*
                if(UAM_Config::$ENV === 'development'){
                    $wpdb->query( " DROP TABLE $table_name"	);
                }*/

        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
		  id   bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
		  time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		  type tinyint NOT NULL,
		  uuid bigint(20) UNSIGNED NOT NULL,
		  ip   tinytext DEFAULT '' NOT NULL,
		  ad_id   bigint(20) UNSIGNED NOT NULL,
		  ad_slide_id   tinyint UNSIGNED NOT NULL,
		  place_id   bigint(20) UNSIGNED NOT NULL,
		  value   int UNSIGNED DEFAULT 1 NOT NULL,
		  UNIQUE KEY id (id)
		) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta($sql);

        $uploads_info = wp_upload_dir();
        $uploads_dir = $uploads_info['basedir'];
        $redirect_dir = "$uploads_dir/uam-redirect"; 
        if(!file_exists($redirect_dir)){
            mkdir($redirect_dir);
            $htaccess_path = $cc_uam_config['assets_dir'] . '/htaccess'; 
            copy($htaccess_path, "$redirect_dir/.htaccess"); 
        }
    }

   




    function legacy(){
//        $pf = get_option( 'cc_phmm_pf' );
        $p = get_option( 'codeneric_uam_prem' );
        global $cc_uam_config;
        if($p && !$cc_uam_config['has_premium_ext']){
            if(!function_exists('cc_uam_base_admin_notice_update_to_premium')){
                function cc_uam_base_admin_notice_update_to_premium() {
                    global $cc_uam_config;
                    $class = "update-nag";
                    $prem_url = admin_url('edit.php');
                    $prem_url = add_query_arg(array('post_type' => $cc_uam_config['custom_post_slug'], 'page' => 'uam-premium'), $prem_url);
                    $p_url =  admin_url('plugins.php');
                    $message = "Please <a id=\"cc_uam_install_notice\" href=\"$prem_url\" >install</a> the Ultimate Ads Manager Premium extension!";
                    echo"<div id=\"cc_uam_notice_wrap\" class=\"$class\"> <p>$message</p></div>";
                }
            }
            add_action( 'admin_notices', 'cc_uam_base_admin_notice_update_to_premium' );

        }
        elseif($cc_uam_config['has_premium_ext'] && !$cc_uam_config['premium_ext_active']){
            if(!function_exists('cc_uam_base_admin_notice_update_to_premium')){
                function cc_uam_base_admin_notice_update_to_premium() {
                    global $cc_uam_config;
                    $class = "update-nag";
                    $d_url =  admin_url('plugins.php');
                    $message = "Please <a href=\"$d_url\" >activate</a> the Ultimate Ads Manager Premium extension!";
                    echo"<div id=\"cc_uam_notice_wrap\" class=\"$class\"> <p>$message</p></div>";
                }
            }

            add_action( 'admin_notices', 'cc_uam_base_admin_notice_update_to_premium' );

        }

    }
}

