<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://codeneric.com
 * @since      1.0.0
 *
 * @package    Ultimate_Ads_Manager
 * @subpackage Ultimate_Ads_Manager/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Ultimate_Ads_Manager
 * @subpackage Ultimate_Ads_Manager/admin
 * @author     Codeneric <contact@codeneric.com>
 */
class Ultimate_Ads_Manager_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */

	public $display_name = 'Ultimate Ads Manager';
	private $version;
	private $stats_calc;
	private $settings_mgr;

	static $settings;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/statistics-calculator.php';
		$this->stats_calc = new Statistics_Calculator();

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/settings.php';
		$this->settings_mgr = new Ultimate_Ads_Manager_Settings($this->plugin_name, $this->version, $this->display_name);
		$this->settings_mgr->load_settings();

		Ultimate_Ads_Manager_Admin::$settings = $this->settings_mgr;


	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Ultimate_Ads_Manager_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Ultimate_Ads_Manager_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style( 'thickbox' );
		
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */


	public function adblock_notice() {  			
    	?>
			<div id="codeneric-uam-adblock-notice" class="notice notice-error">
				<h2>Ultimate Ads Manager</h2>
				<p>It seems like your AdBlocker is conflicting with the plugin. Please whitelist your domain</p>
			</div>
   	 	<?php
	}


	public function generate_js_globals() {
		global $cc_uam_config;
		$globals = array();

		$globals['ad_group'] = $cc_uam_config['plugin_adblock_url'].'/assets/';
		$globals['id'] = get_option($cc_uam_config['plugin_id_option']);

		$globals['wpps_url'] = str_replace('http://', 'https://', $cc_uam_config['wpps_url']);
		$globals['admin_email'] = get_option('admin_email');
		$globals['assets'] = $cc_uam_config['plugin_adblock_url'].'/assets/';


		// paypal_return
		$isSecure =
			( ! empty( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] !== 'off' )
			|| $_SERVER['SERVER_PORT'] == 443;

		$return_url = "$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		$return_url = $isSecure ? "https://$return_url" : "http://$return_url";
		$return_url = add_query_arg( 'paid', 'yes', $return_url );

		$globals['paypal_return'] = $return_url;

		$globals['paypal_return'] = $return_url;
		$globals['paypal_merchant'] = $cc_uam_config['paypal_merchant'];
		$globals['paypal_post_url'] = $cc_uam_config['paypal_post_url'];
		$globals['paypal_env'] = $cc_uam_config['paypal_env'];
		$globals['plugin_display_name'] = $cc_uam_config['plugin_display_name'];
		$globals['checkout_url'] = $cc_uam_config['checkout_url'];



		$globals['stripe_key'] = $cc_uam_config['stripe_key'];


		$post = get_post();


		if($post) {
			$adID = $post->ID;

			$globals['ad_id'] = $adID;

			$globals['ad_meta'] = (array) get_post_meta( $adID , 'ad', true );
            $r = get_post_meta( $adID , 'restrictions', true );
			$globals['ad_restrictions'] = is_array($r) ? $r : array();
            if(!empty($globals['ad_restrictions']['start'])){
                $globals['ad_restrictions']['start'] = date('Y-m-d', $globals['ad_restrictions']['start']);
            }
            if(!empty($globals['ad_restrictions']['end'])){
                $globals['ad_restrictions']['end'] = date('Y-m-d', $globals['ad_restrictions']['end']);
            }
		}



		$adminurl = admin_url( 'edit.php' );
		$globals['support_url'] = add_query_arg( array('post_type' => $cc_uam_config['custom_post_slug'], 'page' => 'codeneric_uam_support'), $adminurl);

//		wp_enqueue_script( 'stripe','https://checkout.stripe.com/checkout.js', array(), '2.1.1', false );

		return $globals;

	}
	public function enqueue_adnotice_removal() {
		global $cc_uam_config;
		wp_enqueue_script("codeneric-uam-adnotice-removal", $cc_uam_config["js_entry_adblock_notice_removal"], array(  ), null, false );
		}

	public function insert_adnotice_fadein_style() {
	 $style =  "
			@keyframes fadein{
				0% { opacity:0; }
				66% { opacity:0; }
				100% { opacity:1; }
			}

			@-webkit-keyframes fadein{
				0% { opacity:0; }
				66% { opacity:0; }
				100% { opacity:1; }
			}
			#codeneric-uam-adblock-notice {
				  animation: 6s ease 0s normal forwards 1 fadein;
			}"; 

		wp_register_style( 'cc-uam-dummy-handle', false );
		wp_enqueue_style( 'cc-uam-dummy-handle' );

		wp_add_inline_style( 'cc-uam-dummy-handle', $style );
	}

	private function trigger_debug_info(){
		$data_changed = get_transient('codeneric/uam/data_changed');
		if($data_changed !== false){
			delete_transient('codeneric/uam/data_changed'); 
			$js = "
				try{
					jQuery.post(ajaxurl, {action: 'codeneric_uam_get_debug_info'}, function(res){window.CODENEIRC_UAM_DEBUG_INFO = res;});		

				}catch(e){
					console.log('debug info failed:', e); 
				}
			"; 
			
		wp_enqueue_script( 'codeneric_uam_request_debug_info', '#', array('jquery'), '1.0' );
   		wp_add_inline_script( 'codeneric_uam_request_debug_info', $js );  

		}
	}
	public function enqueue_ad_script($script_name, $path, $version) {
		$current = get_current_screen();
		global $cc_uam_config;

		if ( $current->base == 'post' && $current->post_type == $cc_uam_config['custom_post_slug'] ) {

			wp_enqueue_media();
			wp_register_script( $script_name, $path, array( 'jquery' ), null, true );

			$post = get_post();
			$adID = $post->ID;
			$meta = (array) get_post_meta( $adID, 'ad', true );

			


			$globals = $this->generate_js_globals();

			wp_localize_script( $script_name, 'UAM_GLOBALS', $globals );
			//wp_localize_script( $script_name, 'codeneric_uam_ad_meta', $meta );
			wp_enqueue_script( $script_name );
			// $this->trigger_debug_info();

		}
	}
	public function enqueue_ad_group_script($script_name, $path, $version) {
		$current = get_current_screen();
		global $cc_uam_config;

		if ( $current->base == 'post' && $current->post_type == $cc_uam_config['custom_post_slug'] . '_group' ) {

			wp_register_script( $script_name, $path, array( 'jquery' ),$version, true );

			$globals = $this->generate_js_globals();

			//-- GET POSTS
			$args        = array(
				'posts_per_page' => - 1,
				'orderby'        => 'date',
				'order'          => 'DESC',
				'post_type'      => $cc_uam_config['custom_post_slug'],
				'post_status'    => 'publish'
				//'suppress_filters' => true
			);
			$posts_array = get_posts( $args );
			$globals['ad_posts'] = $posts_array;

			// -----------------

			$post = get_post();
			$adID = $post->ID;
			$ad_group_ids       = get_post_meta( $adID, 'ad_group', true );
			$ad_group_ids = is_string($ad_group_ids) ? $ad_group_ids : '';
			$ad_group_ids_array = explode( ",", $ad_group_ids );

			if ( $ad_group_ids == "" ) {
				$ad_group_ids_array = array();
			}


			$ad_group = array();
			foreach ( $ad_group_ids_array as $ID ) {
				$temp = get_post( $ID );
				if($temp !== null)
					array_push( $ad_group, $temp );
			}

			$globals['ad_group'] = $ad_group;


			wp_localize_script( $script_name, 'UAM_GLOBALS', $globals );

			wp_enqueue_script( $script_name );
			// $this->trigger_debug_info();


		}
}	


	
	public function register_gutenberg_blocks() {

		return;

		// global $cc_uam_config;
		
		
			
		// wp_register_script("codeneric-uam-places-gutenberg", $cc_uam_config['js_entry_gutenberg_place'], array("wp-blocks","wp-element"),null,true);
		// wp_enqueue_script( "codeneric-uam-places-gutenberg");
		// register_block_type("codeneric/uam-places-block", array(
		// 	"editor_script" => "codeneric-uam-places-gutenberg"			
		// ));

		

	
	}

	

	public function enqueue_settings_script($script_name, $path, $version) {
		$current = get_current_screen();
		global $cc_uam_config;

		if ( $current->base == $cc_uam_config['custom_post_slug'] . '_page_settings' ) {
			wp_register_script( $script_name, $path, array( 'jquery' ),$version, true );

			$globals =  $this->generate_js_globals();

			$globals['settings'] = $this->settings_mgr->general_settings;
			$globals['ajax_url'] = admin_url('admin-ajax.php');
			$next_optimization_job = 'Never';
			$job_time = wp_next_scheduled( $cc_uam_config['optimize_cron_hook'] );
			$db_opt_enabled = isset($globals['settings']) &&
                isset($globals['settings']['db_optimization']) &&
                !empty($globals['settings']['db_optimization']['enabled']);

			if($job_time !== false && $db_opt_enabled){
			    $next_optimization_job = gmdate("F j, Y, g:i a", $job_time);
            }
            $globals['next_optimization_job'] = $next_optimization_job;

            $globals = apply_filters('codeneric/uam/settings_globals', $globals);

			wp_localize_script( $script_name, 'UAM_GLOBALS', $globals );
			//wp_localize_script( $script_name, 'codeneric_uam_ad_meta', $meta );
			wp_enqueue_script( $script_name);
			$this->trigger_debug_info();


		}
	}

    public function enqueue_async_overview_script($script_name, $path, $version) {
        $current = get_current_screen();
        global $cc_uam_config;

        if ( $current->base === 'edit' && $current->post_type === $cc_uam_config['custom_post_slug'] ) {

            wp_register_script( $script_name, $path, array( ),$version, true );
            //wp_localize_script( $script_name, 'codeneric_uam_ad_meta', $meta );
            wp_enqueue_script( $script_name);

        }
    }


	public function enqueue_premium_page_script($script_name, $path, $version) {
		$current = get_current_screen();
		global $cc_uam_config;

		if ( $current->base == $cc_uam_config['custom_post_slug'] . '_page_uam-premium' ) {
			wp_register_script( $script_name, $path, array( 'jquery' ),$version, true );

			$globals =  $this->generate_js_globals();

			$globals['settings'] = $this->settings_mgr->general_settings;

			$globals['has_premium_ext'] = $cc_uam_config['has_premium_ext'];

			$adminurl = admin_url( 'edit.php' );
			$globals['support_url'] = add_query_arg( array('post_type' => $cc_uam_config['custom_post_slug'], 'page' => 'codeneric_uam_support'), $adminurl);


			wp_localize_script( $script_name, 'UAM_GLOBALS', $globals );
			//wp_localize_script( $script_name, 'codeneric_uam_ad_meta', $meta );
			wp_enqueue_script( $script_name);

		}
	}

	public function register_statistics_script($script_name, $path, $version) {
		$current = get_current_screen();
		global $cc_uam_config;

		if ( $current->base == $cc_uam_config['custom_post_slug'] . '_page_statistics' ) {
		
            do_action('codeneric_uam_update_symlink');
			//----------
			wp_register_script( $script_name, $path, array(),$version, true );


			$globals =  $this->generate_js_globals();


			// -- AD POSTS
			$args        = array(
				'orderby'     => 'date',
				'order'       => 'DESC',
				'post_type'   => $cc_uam_config['custom_post_slug'],
				'post_status' => 'publish',
				'numberposts' => -1
				//'suppress_filters' => true
			);

			$posts_array = get_posts( $args );
			$globals['ad_posts'] = $posts_array;

			//----------

			$globals['ad_link'] = add_query_arg( array('post_type' => $cc_uam_config['custom_post_slug']), admin_url('post-new.php') );

			wp_localize_script( $script_name, 'UAM_GLOBALS', $globals );

			//----------

			$this->settings_mgr->load_settings();

			//----------

			//$timezone = isset($this->settings_mgr->general_settings['block_timezone']) ?
			//	$this->settings_mgr->general_settings['block_timezone'] : 'America/Los_Angeles';
			//wp_localize_script( $this->plugin_name.'_statistics', 'codeneric_ad_timezone', $timezone);


			// give initial data
			$uam_statistics_initalquery = '{"type":"click","metric":"unique","history":"last_hour"}';

			if ( isset( $_COOKIE['uam_statistics_initalquery'] ) ) {
				$uam_statistics_initalquery = $_COOKIE['uam_statistics_initalquery'];
				$uam_statistics_initalquery = str_replace( '\\', '', $uam_statistics_initalquery );
			}


			$query = json_decode( $uam_statistics_initalquery, true );
			//print_r($query);
			if ( ! isset( $query['ad_id'] ) ) {
				$args = array(
					'numberposts'      => 1,
					'orderby'          => 'post_date',
					'order'            => 'DESC',
					'post_type'        => $cc_uam_config['custom_post_slug'],
					'post_status'      => 'publish',
					'suppress_filters' => true
				);

				$recent_posts   = wp_get_recent_posts( $args, ARRAY_A );
				$query['ad_id'] = count( $recent_posts ) > 0 ? $recent_posts[0]['ID'] : - 1;

			}

			$gen_set   = get_option( $cc_uam_config['custom_post_slug'] . '_general_settings', array() );
			$client_tz = ! empty( $gen_set['block_timezone'] ) ? $gen_set['block_timezone'] : 'UTC';

			// $tz = date_default_timezone_get();
			// date_default_timezone_set( $client_tz );
			$query['from'] = time();
			$t = time();
			$query['ranges'] = array(
				array($t - (60*60 - 60*15*-1), $t - (60*60 - 60*15*0)),
				array($t - (60*60 - 60*15*0), $t - (60*60 - 60*15*1)),
				array($t - (60*60 - 60*15*1), $t - (60*60 - 60*15*2)),
				array($t - (60*60 - 60*15*2), $t - (60*60 - 60*15*3)),
				array($t - (60*60 - 60*15*3), $t - (60*60 - 60*15*4)),
				 ) ;  
			$query['name'] = 'last_hour'; 
			// date_default_timezone_set( $tz ); 

			$init_data = $this->handle_statistics_query( $query );


			wp_localize_script( $script_name, 'uam_statistics_initaldata', $init_data );
			wp_localize_script( $script_name, 'uam_statistics_initalquery', $query );


			$fakeData                  = json_decode( "{\"last_hour\":{\"unique\":{\"click\":[{\"from\":0,\"to\":1432840415,\"data\":\"8\"}],\"view\":[{\"from\":0,\"to\":1432840415,\"data\":\"2\"}]},\"total\":{\"click\":[{\"from\":0,\"to\":1432840415,\"data\":\"19\"}],\"view\":[{\"from\":0,\"to\":1432840415,\"data\":\"9\"}]}}}" );
			$query_all_time            = $query;
			$query_all_time['history'] = 'last_hour';
			$query_all_time['name'] = 'last_hour';
			$query_all_time['type']    = array( 'click', 'view' );
			$query_all_time['metric']  = array( 'total', 'unique' );
			$query_all_time['ranges'] = array(array(time() - HOUR_IN_SECONDS , time()) ) ;

			$all_time_data = $this->handle_statistics_query( $query_all_time );
			//$all_time_data = $fakeData;
			$all_time_data = json_decode( $all_time_data );


			wp_localize_script( $script_name, 'uam_statistics_initaloverview', $all_time_data );


			return $query['ad_id'];

		}
	}

	public function enqueue_scripts() {

		$current = get_current_screen();
		global $cc_uam_config;
        do_action('codeneric_uam_update_symlink');


		$this->register_gutenberg_blocks();
		$this->register_statistics_script($this->plugin_name.'_statistics', $cc_uam_config['js_entry_statistics'],$this->version);


		wp_enqueue_script($this->plugin_name.'_statistics'); // exported so we can add more stuff in prem

		 $this->enqueue_ad_script($this->plugin_name,$cc_uam_config['js_entry_edit'], $this->version );

		 $this->enqueue_ad_group_script($this->plugin_name.'_group',$cc_uam_config['js_entry_group_edit'], $this->version );
		$this->enqueue_adnotice_removal();

		$this->enqueue_settings_script($this->plugin_name.'_settings', $cc_uam_config['js_entry_settings'],$this->version);

		$this->enqueue_premium_page_script($this->plugin_name.'_premium-page', $cc_uam_config['js_entry_premium'],$this->version);
        $this->enqueue_async_overview_script($this->plugin_name.'_async-overview', $cc_uam_config['js_entry_fetch_async_overview'],$this->version);



	}


//	public function define_table_columns($column_name, $id) {
	public function define_table_columns( $column_name ) {
		//die($column_name);
		$cols = array(
			'cb'            => '<input type="checkbox" />',
			'title'         => __( 'Title' ),
			'shortcode'     => __( 'Shortcode' ),
			'date'          => __( 'Date' ),
			'total_views'   => __( 'Total Views' ),
			'unique_views'  => __( 'Unique Views' ),
			'total_clicks'  => __( 'Total Clicks' ),
			'unique_clicks' => __( 'Unique Clicks' ),
		);

		return $cols;
	}

    public function define_places_table_columns( $column_name ) {
        //die($column_name);
        $cols = array(
            'cb'            => '<input type="checkbox" />',
            'title'         => __( 'Title' ),
            'shortcode'     => __( 'Shortcode' ),
            'date'          => __( 'Date' ),
        );

        return $cols;
    }
    public function fill_places_custom_columns( $column, $post_id ) {
        if ( ! isset( $post_id ) ) {
            return;
        }

        switch ( $column ) {
            case "shortcode":
                echo '[uam_place id="' . $post_id . '"]';
                break;
        }
    }

	public function fill_custom_columns( $column, $post_id ) {
		if ( ! isset( $post_id ) ) {
			return;
		}
		require_once dirname( __FILE__ ) . "/../includes/statistics-calculator.php";
		$stat_calc = new Statistics_Calculator();
		//$add = get_post_meta($post_id,"codeneric_ad",true);

		switch ( $column ) {
			case "shortcode":
				echo '[uam_ad id="' . $post_id . '"]';
				break;
			case "total_views":
//				echo $stat_calc->get_total_events( $post_id, 'view' );
                echo "<div data-codeneric-uam-type='view' data-codeneric-uam-metric='total' data-codeneric-uam-history='all_time' data-codeneric-uam-id='$post_id' class='codeneric-uam-async-request'>...</div>";
				break;
			case "unique_views":
//				echo $stat_calc->get_unique_events( $post_id, 'view' );
                echo "<div data-codeneric-uam-type='view' data-codeneric-uam-metric='unique' data-codeneric-uam-history='all_time' data-codeneric-uam-id='$post_id' class='codeneric-uam-async-request'>...</div>";
                break;
			case "total_clicks":
//				echo $stat_calc->get_total_events( $post_id, 'click' );
                echo "<div data-codeneric-uam-type='click' data-codeneric-uam-metric='total' data-codeneric-uam-history='all_time' data-codeneric-uam-id='$post_id' class='codeneric-uam-async-request'>...</div>";
                break;
			case "unique_clicks":
//				echo $stat_calc->get_unique_events( $post_id, 'click' );
                echo "<div data-codeneric-uam-type='click' data-codeneric-uam-metric='unique' data-codeneric-uam-history='all_time' data-codeneric-uam-id='$post_id' class='codeneric-uam-async-request'>...</div>";
                break;
		}
	}

	public function define_table_columns_group( $column_name ) {
		//die($column_name);
		$cols = array(
			'cb'        => '<input type="checkbox" />',
			'title'     => __( 'Title' ),
			'shortcode' => __( 'Shortcode' ),
			'date'      => __( 'Date' )
		);

		return $cols;
	}

	public function fill_custom_columns_group( $column, $post_id ) {
		if ( ! isset( $post_id ) ) {
			return;
		}


		switch ( $column ) {
			case "shortcode":
				echo '[uam_ad id="' . $post_id . '"]';
				break;
		}
	}

	public function register_post_type_codeneric_ad() {
		global $cc_uam_config;
		register_post_type( $cc_uam_config['custom_post_slug'],
			array(
				'labels'              => array(
					'menu_name'     => $cc_uam_config['plugin_display_name'],
					'all_items'     => __( 'Adverts' ),
					'name'          => __( 'Adverts' ),
					'singular_name' => __( 'Advert' ),
					'edit_item'     => __( 'Edit Advert' ),
					'new_item'      => __( 'New Advert' ),
					'add_new_item'  => __( 'Add New Advert' )
				),
				'public'              => true,
				'publicly_queryable'  => false,
				'show_ui'             => true,
				'query_var'           => true,
				'can_export'          => true,
				'exclude_from_search' => true,
				'has_archive'         => false,
				'menu_icon'           => 'dashicons-welcome-view-site',
				'rewrite'             => array( 'slug' => $cc_uam_config['custom_post_slug'], 'with_front' => false ),
				'supports'            => array(
					'title',
					'editor' => false
				),
				'taxonomies'          => array( 'category' ),
			)
		);
		register_post_type( $cc_uam_config['custom_post_slug'] . '_group',
			array(
				'labels'             => array(
					'name'          => __( 'Groups' ),
					'singular_name' => __( 'Group' ),
					'edit_item'     => __( 'Edit  Group' ),
					'new_item'      => __( 'New Group' ),
					'add_new_item'  => __( 'Add New Group' )
				),
				'public'             => true,
				'publicly_queryable' => true,
				'show_ui'            => true,
				'query_var'          => true,
				'can_export'         => true,
				'has_archive'        => false,
				'show_in_menu'       => 'edit.php?post_type=codeneric_ad',
				'rewrite'            => array( 'slug'       => $cc_uam_config['custom_post_slug'] . '_group',
				                               'with_front' => false
				),
				'supports'           => array(
					'title',
					'editor' => false
				),
				'taxonomies'         => array( 'category' )

			)
		);
        register_post_type( $cc_uam_config['custom_post_slug'] . '_place',
            array(
                'labels'             => array(
                    'name'          => __( 'Places' ),
                    'singular_name' => __( ' Place' ),
                    'edit_item'     => __( 'Edit  Place' ),
                    'new_item'      => __( 'New Place' ),
                    'add_new_item'  => __( 'Add New Place' )
                ),
                'public'             => true,
                'publicly_queryable' => true,
                'show_ui'            => true,
                'query_var'          => true,
                'can_export'         => true,
                'has_archive'        => false,
                'show_in_menu'       => 'edit.php?post_type='.$cc_uam_config['custom_post_slug'],
                'rewrite'            => array( 'slug'       => $cc_uam_config['place_slug'],
                    'with_front' => false
                ),
                'supports'           => array(
                    'title',
                    'editor' => false
                ),
                'taxonomies'          => array( 'category' )
//                'taxonomies'         => array( '' )

            )
        );
	}

	public function hide_add_new() {
        global $submenu, $cc_uam_config;
        unset($submenu['edit.php?post_type='.$cc_uam_config['custom_post_slug']][10]);
//        remove_submenu_page('edit.php', 'edit-tags.php?taxonomy=category&post_type='.$cc_uam_config['custom_post_slug']);
//        remove_submenu_page('edit.php', 'edit-tags.php?taxonomy=category&post_type=codeneric_ad');
        $post_type = $cc_uam_config['custom_post_slug'];
        $tax_slug = 'manage_categories';
        if (isset($submenu['edit.php?post_type='.$post_type])) {
            foreach ($submenu['edit.php?post_type=' . $post_type] as $k => $sub) {
                if ($sub[1] === $tax_slug) {
                    unset($submenu['edit.php?post_type=' . $post_type][$k]);
                }

            }
        }
	}

	public function add_meta_boxes() {
		global $cc_uam_config;
		
		$this->trigger_debug_info();

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-meta.php';
		add_meta_box( $this->plugin_name . '_meta', 'Ad', 'codeneric_ad_manager_meta', $cc_uam_config['custom_post_slug'], 'normal', 'high' );


		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-meta-group.php';
		add_meta_box( $this->plugin_name . '_meta_group', 'Group', 'codeneric_ad_manager_meta_group', $cc_uam_config['custom_post_slug'] . '_group', 'normal', 'high' );

        require_once  dirname(dirname( __FILE__ ) ) . '/admin/partials/place.php';
        add_meta_box( $this->plugin_name . '_place', 'Place', 'codeneric_ad_manager_place', $cc_uam_config['place_slug'], 'normal', 'high' );

    }

	public function save_meta_boxes( $post_id ) {
		/*
		 * We need to verify this came from the our screen and with proper authorization,
		 * because save_post can be triggered at other times.
		 */
		global $cc_uam_config;

        require_once dirname(__FILE__).'/../includes/place.php';

		// Check if our nonce is set.
		if ( ! isset( $_POST[ $cc_uam_config['wp_nonce_base'] . '_nonce' ] ) ) {
			return $post_id;
		}

		$nonce = $_POST[ $cc_uam_config['wp_nonce_base'] . '_nonce' ];

		// Verify that the nonce is valid.
		if ( ! wp_verify_nonce( $nonce, $cc_uam_config['wp_nonce_base'] ) ) {
			return $post_id;
		}


		// If this is an autosave, our form has not been submitted,
		//     so we don't want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}

		// Check the user's permissions.
		if ( $cc_uam_config['custom_post_slug'] == $_POST['post_type'] || $cc_uam_config['custom_post_slug'] . '_group' == $_POST['post_type'] ) {

			if ( ! current_user_can( 'edit_page', $post_id ) ) {
				return $post_id;
			}

		} else {

			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return $post_id;
			}
		}

		if ( $cc_uam_config['custom_post_slug'] == $_POST['post_type'] ) {
            require_once dirname(__FILE__).'/../includes/util.php';

			// Sanitize the user input.
            if(isset($_POST['ad'] ) && isset($_POST['ad']['type'])){
                if($_POST['ad']['type'] === 'image_ad'){
                    $image_id = $_POST['ad']['image_ad_id'];
                    $src = wp_get_attachment_image_src( $image_id, 'full', false );
                    $width = $src[1];
                    $height = $src[2];
                    $form = Ultimate_Ads_Manager_Util::nearest_form($width,$height);
                    $_POST['ad']['forms'] = array($form);
                }else{
                    $width = 1; //TODO: let them select forms in frontend. For now we do not have any preference
                    $height = 1;
                    $form = Ultimate_Ads_Manager_Util::nearest_form($width,$height);
                    $_POST['ad']['forms'] = array($form);
                }
            }


			//$mydata = sanitize_text_field( $_POST['ad']['referral_url'] );

			//die(var_dump($_POST['ad']));

            if(isset($_POST['restrictions'])){
                if(!empty($_POST['restrictions']['start'])){
                    $_POST['restrictions']['start'] = strtotime($_POST['restrictions']['start']);
                }

                if(!empty($_POST['restrictions']['end'])){
                    $_POST['restrictions']['end'] = strtotime($_POST['restrictions']['end']);
                }
                update_post_meta( $post_id, 'restrictions', $_POST['restrictions'] );
            }

            do_action('codeneric/uam/pre_save_ad', $_POST['ad'] );
			update_post_meta( $post_id, 'ad', $_POST['ad'] );
            do_action('codeneric/uam/after_save_ad', $post_id, $_POST['ad'] );

			\set_transient('codeneric/uam/data_changed', true, 60);


		}

		if ( $cc_uam_config['group_slug']  == $_POST['post_type'] ) {
			if ( ! isset( $_POST['ad_group'] ) ) {
				$_POST['ad_group'] = array();
			}
			//$_POST['ad_group']['ids'] = isset($_POST['ad_group']['ids']) ? explode(',',$_POST['ad_group']['ids']) : array();
			//print_r($_POST['ad_group']); //TODO: fix this issue (#4)
			//die();


			// just save the id dude
//			$ad_group = json_decode(urldecode($_POST['ad_group']));
//
//			// get rid of label
//			foreach ($ad_group as $ad) {
//				unset($ad->label);
//			}

            if(isset($_POST['restrictions'])){
                update_post_meta( $post_id, 'restrictions', $_POST['restrictions'] );
            }

            do_action('codeneric/uam/pre_save_group', $_POST['ad_group'] );
            update_post_meta( $post_id, 'ad_group', $_POST['ad_group'] );
            do_action('codeneric/uam/after_save_group', $post_id);
			\set_transient('codeneric/uam/data_changed', true, 60);

		}

		return $post_id;
	}


    public function save_place( $post_id ) {
        /*
         * We need to verify this came from the our screen and with proper authorization,
         * because save_post can be triggered at other times.
         */
        global $cc_uam_config;

        // Check if our nonce is set.
        if ( ! isset( $_POST[ $cc_uam_config['wp_nonce_base'] . '_nonce' ] ) ) {
            return $post_id;
        }

        $nonce = $_POST[ $cc_uam_config['wp_nonce_base'] . '_nonce' ];

        // Verify that the nonce is valid.
        if ( ! wp_verify_nonce( $nonce, $cc_uam_config['wp_nonce_base'] ) ) {
            return $post_id;
        }


        // If this is an autosave, our form has not been submitted,
        //     so we don't want to do anything.
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        // Check the user's permissions.
        if ( $cc_uam_config['place_slug'] == $_POST['post_type']  ) {

            if ( ! current_user_can( 'edit_page', $post_id ) ) {
                return $post_id;
            }

        } else {

            if ( ! current_user_can( 'edit_post', $post_id ) ) {
                return $post_id;
            }
        }

        if ( $cc_uam_config['place_slug'] == $_POST['post_type'] ) {
            // Sanitize the user input.

            if(isset($_POST['forms']) && is_array($_POST['forms'])){
                $forms = array_keys($_POST['forms']);
                update_post_meta( $post_id, 'forms', $forms );
                $meta = get_post_meta( $post_id, 'forms', true );
            }

            require_once dirname(__FILE__).'/../includes/place.php';
            Ultimate_Ads_Manager_Place::update_place($post_id);
			\set_transient('codeneric/uam/data_changed', true, 60);


        }

        return $post_id;
    }


	public function add_submenu_pages() {
		global $cc_uam_config;
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-statistics.php';
		add_submenu_page( 'edit.php?post_type=' . $cc_uam_config['custom_post_slug'], $this->display_name . ' ' . __( 'Statistics' ), __( 'Statistics' ), 'manage_options', 'statistics', 'ultimate_ads_manager_statistics_page' );

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-settings.php';
		add_submenu_page( 'edit.php?post_type=' . $cc_uam_config['custom_post_slug'], $this->display_name . ' ' . __( 'Settings' ), __( 'Settings' ), 'manage_options', 'settings', 'codeneric_ad_manager_settings_page' );

//		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-help.php';
//		add_submenu_page( 'edit.php?post_type=' . $cc_uam_config['custom_post_slug'], $this->display_name . ' ' . __( 'Help' ), __( 'Help' ), 'manage_options', 'help', 'codeneric_ad_manager_help_page' );

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-support.php';
		add_submenu_page( 'edit.php?post_type=' . $cc_uam_config['custom_post_slug'], $this->display_name . ' ' . __( 'Support' ), __( 'Support' ), 'manage_options', 'codeneric_uam_support', 'codeneric_ad_manager_support_page' );


		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/admin-premium.php';
		add_submenu_page( 'edit.php?post_type=' . $cc_uam_config['custom_post_slug'], $this->display_name . ' ' . __( 'Premium' ), __( 'Premium' ), 'manage_options', 'uam-premium', 'codeneric_ad_manager_premium_page' );


	}

	public function send_support_feedback() {
		global $cc_uam_config;
		if ( empty( $_POST ) ||
		     ( ! isset( $_POST['cc_send_feedback_nonce'] ) ) ||
		     ( ! wp_verify_nonce( $_POST['cc_send_feedback_nonce'], 'cc_send_feedback' )
		     )
		) {
			echo 'You targeted the right function, but sorry, your nonce did not verify.';
			die();
		} else {


			$data      = $_POST['support'];
			$to        = array( 'support@codeneric.com' );
			$subject   = 'UAM: '.$data['subject'];
			$headers[] = 'From: <' . $data['email'] . '>';

			$message = $data['content'];
			if ( isset( $_POST['cc_send_feedback_nonce'] ) ) {
				if ( wp_mail( $to, $subject, $message, $headers ) ) {
					wp_redirect( admin_url() . 'edit.php?post_type='.$cc_uam_config['custom_post_slug'].'&page=codeneric_uam_support&is_send=true' );
				} else {
					wp_redirect( admin_url() . 'edit.php?post_type='.$cc_uam_config['custom_post_slug'].'&page=codeneric_uam_support&is_send=false' );
				}
			}
		}
	}

	public function handle_statistics_query( $q ) {

		// $query = isset( $_POST['query'] ) ? json_decode($_POST['query']) : $q;
		$query = isset( $_POST['query'] ) ? $_POST['query'] : $q;
		//print_r($query);exit;
		if ( ! isset( $query ) ||
            ! isset( $query['type'] ) ||
            (!isset( $query['ad_id'] ) && !isset( $query['place_id'] ))
		) {
			status_header( 400 );
			exit('Bad Ultimate Ads Manager query!');
		}
		if (! isset( $query['name']) ) {
			status_header( 400 );
			exit('Property "name" has to be defined in queries since Ultimate Ads Manager 2.3.0!'); 
		}
		if (! isset( $query['ranges']) ) {
			status_header( 400 );
			exit('Property "ranges" has to be defined in queries since Ultimate Ads Manager 2.3.0!'); 
		}


		$query['ad_slide_id'] = isset( $query['ad_slide_id'] ) ? $query['ad_slide_id'] : null;
		$query['ad_id'] = isset( $query['ad_id'] ) ? $query['ad_id'] : null;
		$query['place_id'] = isset( $query['place_id'] ) ? $query['place_id'] : null;
		$query['metric']      = isset( $query['metric'] ) ? $query['metric'] : array();
		$query['metric']      = is_array( $query['metric'] ) ? $query['metric'] : array( $query['metric'] );
		if ( isset( $query['metric'] ) && ! is_array( $query['metric'] ) ) {
			$query['metric'] = array( $query['metric'] );
		}

		if ( ! isset( $query['metric'] ) ) {
			$query['metric'] = array();
		}

		if ( ! is_array( $query['type'] ) ) {
			$query['type'] = array( $query['type'] );
		}


		$hist = $query['history'];
		$res  = array( $hist => array() );
		//$res['metric'] = array();
		//$res['metric'];

		foreach ( $query['metric'] as $metric ) {
			$res[ $hist ][ $metric ] = array();
			foreach ( $query['type'] as $type ) {
				$temp_query           = $query;
				$temp_query['type']   = $type;
				$temp_query['metric'] = $metric;

				$res[ $hist ][ $metric ][ $type ] = array();
				$tn = $temp_query['name'];
				if(in_array($tn, array('ctr', 'custom_range', 'last_7_days', 'last_12_months'))){ 
					$res[$hist][$metric][$type] = apply_filters('codeneric/uam/premium/statistics/query', $temp_query);
				}else{
					$res[$hist][$metric][$type] = $this->stats_calc->query( $temp_query );

				}
			}
		}


		if ( isset( $_POST['query'] ) ) {
			header( "Content-Type: application/json" );
			$json = json_encode( $res );
			exit( json_encode( $json ) );
		} else {
			$json = json_encode( $res );

			return $json;
		}


	}

	public function proxy_get() {
		$url = $_POST['url'];
		$res = wp_remote_get( $url );
		if ( empty( $res ) ) {
			status_header( 500 );
			exit;
		}
		status_header( $res['response']['code'] );
		exit( $res['body'] );
	}


	public function register_shortcodes() {
		require_once dirname( __FILE__ ). '/../includes/shortcode.php';

		Ultimate_Ads_Manager_Base_Shortcode::init();
	}

	public function listen_on_call(){
		$result = $_POST['bool'] == 'true';
		delete_option('__temp_site_transiant_5448uam4886');
		update_option('codeneric_uam_prem', $result);
	}

	private function rrmdir($src) {
    $dir = opendir($src);
    while(false !== ( $file = readdir($dir)) ) {
        if (( $file != '.' ) && ( $file != '..' )) {
            $full = $src . '/' . $file;
			if(is_link($full)){
				unlink($full); 
			}
            elseif ( is_dir($full) ) {
                rrmdir($full);
            }
            else {
                unlink($full);
            }
        }
    }
    closedir($dir);
    rmdir($src);
}

	public function update_symlink(){  
        // global $cc_uam_config;
        // $has_premium_ext = $cc_uam_config['has_premium_ext'];

		// $uam_redirect_parent_folder = $cc_uam_config['uam_redirect_parent_folder'];
		// $old_uam_redirect_parent_folder = get_option('update_symlink/uam_redirect_parent_folder', $uam_redirect_parent_folder);
		// update_option('update_symlink/uam_redirect_parent_folder', $uam_redirect_parent_folder); 

		// if($uam_redirect_parent_folder !== $old_uam_redirect_parent_folder){ //folder outdated! delete the old one
		// 	if(is_dir($old_uam_redirect_parent_folder)){
		// 		$this->rrmdir($old_uam_redirect_parent_folder); 
		// 	} 
		// }   

		// if(!file_exists($cc_uam_config['uam_redirect_parent_folder'])){
		// 	mkdir($cc_uam_config['uam_redirect_parent_folder']); 
		// }

        // if(!is_link($cc_uam_config['plugin_adblock_symlink_path'])
        // && file_exists($cc_uam_config['plugin_adblock_symlink_path'])){
        //     unlink($cc_uam_config['plugin_adblock_symlink_path']);
        // }

        // if(is_link($cc_uam_config['plugin_adblock_symlink_path']) ){
        //     if(readlink($cc_uam_config['plugin_adblock_symlink_path']) !== $cc_uam_config['plugin_root_path']){
        //         unlink($cc_uam_config['plugin_adblock_symlink_path']);
        //         symlink($cc_uam_config['plugin_root_path'], $cc_uam_config['plugin_adblock_symlink_path']);
        //     }
        // }

        // if(!is_link($cc_uam_config['plugin_adblock_symlink_path'])
        //     && !file_exists($cc_uam_config['plugin_adblock_symlink_path'])){
        //     symlink($cc_uam_config['plugin_root_path'], $cc_uam_config['plugin_adblock_symlink_path']);
        // }

        // if($has_premium_ext){
        //     if(!is_link($cc_uam_config['premium_plugin_adblock_symlink_path'])
        //         && file_exists($cc_uam_config['premium_plugin_adblock_symlink_path'])){
        //         unlink($cc_uam_config['premium_plugin_adblock_symlink_path']);
        //     }

        //     if(is_link($cc_uam_config['premium_plugin_adblock_symlink_path']) ){
        //         if(readlink($cc_uam_config['plugin_adblock_symlink_path']) !== $cc_uam_config['premium_plugin_root_path']){
        //             unlink($cc_uam_config['premium_plugin_adblock_symlink_path']);
        //             symlink($cc_uam_config['premium_plugin_root_path'], $cc_uam_config['premium_plugin_adblock_symlink_path']);
        //         }
        //     }

        //     if(!is_link($cc_uam_config['premium_plugin_adblock_symlink_path'])
        //         && !file_exists($cc_uam_config['premium_plugin_adblock_symlink_path'])){
        //         symlink($cc_uam_config['premium_plugin_root_path'], $cc_uam_config['premium_plugin_adblock_symlink_path']);
        //     }
        // }
    }


    public function update_places_after_advert($ad_id, $ad){
        require_once dirname(__FILE__).'/../includes/place.php';
        Ultimate_Ads_Manager_Place::after_save_ad($ad_id, $ad);
    }

    public function update_places_after_group($ad_id){
        require_once dirname(__FILE__).'/../includes/place.php';
        Ultimate_Ads_Manager_Place::after_save_group($ad_id);
    }

    public function update_places_on_deletion($id){
        require_once dirname(__FILE__).'/../includes/place.php';
        Ultimate_Ads_Manager_Place::after_candidate_deleted($id);
    }

    public function update_groups_on_deletion($id){
        require_once dirname(__FILE__).'/../includes/group.php';
        Ultimate_Ads_Manager_Place::after_save_group($id);
    }

    public function register_callback($args){
        require_once dirname(__FILE__).'/../includes/common.php';
        $in = isset($args['in']) ?  $args['in'] : 0;
        $singleton = isset($args['singleton']) ?  $args['singleton'] : false;
        Ultimate_Ads_Manager_Base_Common::register_callback($args['url'], $in, $singleton);
    }

	public function handle_legacy(){
		require_once dirname(__FILE__).'/../includes/legacy.php';  
		Ultimate_Ads_Manager_Legacy::handle();
	}

	public function admin_notice_recommend_premium(){
		global $cc_uam_config, $wpdb;
        $has_premium_ext = $cc_uam_config['has_premium_ext']; 
		if($has_premium_ext)return; 
		$id = "cc-uam-notice-recommend-premium";
		$cool_down_transient = "$id-cooldown";
		$cool_down = 60*60*24*7; //7 days
		$cool_down_exceeded = get_transient($cool_down_transient) === false;
		if(!$cool_down_exceeded)return; 

		$res = get_transient("$id-db-query");
		if($res === false){
			$table_name_events = $cc_uam_config['table_name_events'];
			$res = $wpdb->get_results("SELECT * FROM $table_name_events ORDER BY id DESC LIMIT 1", ARRAY_A);
			set_transient("$id-db-query", $res, 60*60*24*1);
		} 
		$has_many_events = count($res) > 0 && $res[0]['id'] > 10 * 1000; 
		if(!$has_many_events)return; 
		
		//check cool down
		$class = "notice-info is-dismissible"; 
		$title = "<h2>Ultimate Ads Manager</h2>";
		$link = "https://codeneric.com/products/ultimate-ads-manager/?utm_source=uam-base-plugin&utm_medium=admin-notice&utm_campaign=recommend-premium#pricing";
		$atag = "<a href=\"$link\">Ultimate Ads Manager Premium</a>"; 
		$message = "Your site has lots of traffic, which is awesome! But we <strong>strongly recommend</strong> to install $atag to keep your ads fast and working!";
		$js = "
			function codeneric_uam_set_transient(id, transient, time){
				jQuery('#'+id+' > button').click(function(){
					console.log('KLICKED');
					jQuery.post(ajaxurl, {action:'codeneric_uam_set_transient', transient: transient, value: 'yes', time: time });
				}); 
			}
			codeneric_uam_set_transient('$id', '$cool_down_transient', $cool_down);
			jQuery('#$id').bind('DOMSubtreeModified',function(event) {
				codeneric_uam_set_transient('$id', '$cool_down_transient', $cool_down); 
			});
			"; 
			
		wp_enqueue_script( 'codeneric_admin_notice_js', '#', array('jquery'), '1.0' );
   		wp_add_inline_script( 'codeneric_admin_notice_js', $js );  
		echo"<div id=\"$id\" class=\"notice $class\"> $title <p>$message</p></div>";

	} 

	public function admin_notice_november_deal(){
		global $cc_uam_config, $wpdb;
        $has_premium_ext = $cc_uam_config['has_premium_ext']; 
		if($has_premium_ext)return; 

		$end_date = strtotime(date('Y-m-d', strtotime('2019-11-30') ) );
		$now = time(); 
		if($now > $end_date)return; 

		$id = "cc-uam-notice-november-deal";
		$cool_down_transient = "$id-cooldown";
		$cool_down = 60*60*24*1; //1 day
		$cool_down_exceeded = get_transient($cool_down_transient) === false;
		if(!$cool_down_exceeded)return; 

		
		//check cool down
		$class = "notice-info is-dismissible"; 
		$title = "<h2>Ultimate Ads Manager November Deal:</h2>";
		$link = "https://codeneric.com/products/ultimate-ads-manager/?utm_source=uam-base-plugin&utm_medium=uam-admin-notice&utm_campaign=uam-recommend-premium#pricing";
		$atag = "<a href=\"$link\">Ultimate Ads Manager Premium</a>"; 
		$message = "Use the coupon code <strong>uam-november-deal</strong> and get <strong>30% off</strong> of $atag. This deal ends this month.";
		$js = "
			function codeneric_uam_set_transient(id, transient, time){
				jQuery('#'+id+' > button').click(function(){
					console.log('KLICKED');
					jQuery.post(ajaxurl, {action:'codeneric_uam_set_transient', transient: transient, value: 'yes', time: time });
				}); 
			}
			codeneric_uam_set_transient('$id', '$cool_down_transient', $cool_down);
			jQuery('#$id').bind('DOMSubtreeModified',function(event) {
				codeneric_uam_set_transient('$id', '$cool_down_transient', $cool_down); 
			});
			"; 
			
		wp_enqueue_script( 'codeneric_admin_notice_js', '#', array('jquery'), '1.0' );
   		wp_add_inline_script( 'codeneric_admin_notice_js', $js );  
		echo"<div id=\"$id\" class=\"notice $class\"> $title <p>$message</p></div>";

	} 

	public function ajax_set_transient(){
		if(
			!isset($_POST['transient']) ||
			!isset($_POST['value']) ||
			!isset($_POST['time']) 
			)wp_die('Parameters transient, value and time have to be set!'); 
		$transient = $_POST['transient'];
		$value = $_POST['value'];
		$time = $_POST['time'];

		$res = set_transient($transient, $value, $time); 
		wp_send_json(array('success' => $res)); 
		wp_die(); 

	}

	public function plugin_deactivation_survey() {
		global $cc_uam_config, $pagenow;
    $config = $cc_uam_config; 
    $R = $_REQUEST;
    $P = $_POST;
    $S = $_SERVER;  


    $plugin = 'ultimate-ads-manager/ultimate-ads-manager.php'; 

    // making sure we're running in plugins.php admin page
    if ($pagenow !== 'plugins.php')
      return;

    // deactivations only
    if (!\array_key_exists('action', $R))
      return;

    // deactivations only
    if (\array_key_exists('action', $R) && 'deactivate' !== $R['action'])
      return;

    // run on our plugin deactivation
    if (!\array_key_exists('plugin', $R))
      return;
    // run on our plugin deactivation
    if (\array_key_exists('plugin', $R) && $plugin !== $R['plugin'])
      return;

    // make sure we don't make plugin deactivations impossible
    if (\array_key_exists('ignore_sv', $R))
      return;


    $ignore_sv_url = (string)/*UNSAFE_EXPR*/ \add_query_arg(
      array('ignore_sv' => 1),
      $S['REQUEST_URI']
    );    

    if (\array_key_exists('submit_sv', $P) && $P['submit_sv'] === '1') {
      $ds_post = $P;
	  $ds_post['allow_contact']	= \array_key_exists('allow_contact', $ds_post); 
	  $ds_post['reason']	= \array_key_exists('reason', $ds_post) ? $ds_post['reason'] : 'UNKNOWN';
	  $ds_post['explanation']	= \array_key_exists('explanation', $ds_post) ? $ds_post['explanation'] : 'UNKNOWN';
      $email = $ds_post['email'];
      $email = !\is_null($email) && $ds_post['allow_contact'] ? $email : 'UNKNOWN';  
        
        $data = array(
          'version' => $config['version'],
          'reason' => \sanitize_text_field($ds_post['reason']),
          'explanation' => \sanitize_text_field($ds_post['explanation']),
          'email' => $email
        );
        $body = array(
          // 'site_url' => \site_url(), 
          'site_url' => 'UNKNOWN', 
          'type' => 'deactivation_survey',
          'context'=> 'uam',
          'version'=> 1,
          'data' => $data
        ); 

        $args = array(
          'body' => $body, 
          'blocking' => false,
          'sslverify' => false
        );
        \wp_remote_post($config['wpps_url'].'/event/2.0/', $args); 

      if (
          $ds_post['allow_contact'] === true && \is_email($ds_post['email'])
        ) {
          $res = \wp_mail(
            'support@codeneric.com',
            'Deactivation Survey',
            \sprintf(
              "%s: %s",
              \sanitize_text_field($ds_post['reason']),
              \sanitize_text_field($ds_post['explanation'])
            ),
            array(
              'From: '.
              \get_bloginfo('name').
              ' <'.
              \sanitize_text_field($email).
              '>'
            )
          );
      } 
      
      
      // refresh the page to continue to plugin deactivation
      \wp_safe_redirect($ignore_sv_url);
      exit();
    } else {

      $deactivation_survey =
        \file_get_contents(__DIR__.'/../assets/deactivation-survey.html');
      $deactivation_survey = \str_replace(
        "%%ADMIN_PLUGINS_URL%%",
        \admin_url('plugins.php'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%ADMIN_EMAIL%%",
        \get_bloginfo('admin_email', 'display'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%JUST_DEACTIVATE_URL%%",
        $ignore_sv_url,
        $deactivation_survey
      );


      $deactivation_survey = \str_replace(
        "%%_I18_Cancel%%",
        \__('Cancel', 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_title%%",
        \__('Please let me know why you are deactivating Ultimate Ads Manager:', 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_couldnt_understand%%",
        \__("couldn't understand how to make it work", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_happy_to_help%%",
        \__("we are happy to help you out! Just drop us a mail:", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_found_better_plugin%%",
        \__("Found a better plugin", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_missing_feature%%",
        \__("Ultimate Ads Manager is missing a feature", 'ultimate-ads-manager'), 
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_not_looking_for%%",
        \__("It's not what I was looking for", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_didnt_work_as_expected%%",
        \__("The plugin didn't work as expected", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_deactivate_temporarily%%",
        \__("Deactivating temporarily to test something", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_other%%",
        \__("Other", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_reach_me_at%%",
        \__("You can reach me at", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_to_discuss%%",
        \__("to discuss in more detail", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_just_deactivate%%",
        \__("I just want to deactivate", 'ultimate-ads-manager'),
        $deactivation_survey
      );
      $deactivation_survey = \str_replace(
        "%%_I18_no_additional_data%%",
        \__("The only information send is your response to this survey. No additional data is collected about you or your website. You will not be added to any email list.", 'ultimate-ads-manager'),
        $deactivation_survey
      );



      \wp_die($deactivation_survey, 'Deactivating Ultimate Ads Manager...');
    }
  }

	public function get_debug_info(){
		global $cc_uam_config, $wpdb;
		$phpversion = \phpversion();
		$wp_version = $GLOBALS['wp_version'];
		$active_plugins = \get_option('active_plugins');
		$plugins = \get_plugins();
		$plugin_id = get_option($cc_uam_config['plugin_id_option']);
		$locale = \get_locale();

		$theme = \wp_get_theme();
		$theme_data = array(
		'Name' => $theme->get('Name'),
		'Version' => $theme->get('Version'),
		'ThemeURI' => $theme->get('ThemeURI'),
		);

		$ad_slug = $cc_uam_config['custom_post_slug'];
		$group_slug = $cc_uam_config['group_slug'];
		$place_slug = $cc_uam_config['place_slug']; 

		$ad_ids_query = [
			'post_type'      => $ad_slug,
			'posts_per_page' => -1,
			'fields'         => 'ids' 
		];
		$ad_ids = get_posts( $ad_ids_query ); 
		$adverts = array();
		foreach($ad_ids as $ad_id){
			$ad_meta = (array) get_post_meta( $ad_id , 'ad', true );
            $r = get_post_meta( $ad_id , 'restrictions', true );
			$r = is_array($r) ? $r : array(); 
			$adverts[] = array('ad_meta' => $ad_meta, 'restrictions' => $r, 'ad_id' => $ad_id);  
		}

		$group_ids_query = [
			'post_type'      => $group_slug,
			'posts_per_page' => -1,
			'fields'         => 'ids' 
		];
		$group_ids = get_posts( $group_ids_query ); 
		$groups = array();
		foreach($group_ids as $group_id){
			$ad_group_ids       = get_post_meta( $group_id, 'ad_group', true );
			$ad_group_ids = is_string($ad_group_ids) ? $ad_group_ids : '';
			$ad_group_ids_array = explode( ",", $ad_group_ids );

            $r = get_post_meta( $group_id , 'restrictions', true );
			$r = is_array($r) ? $r : array(); 
			$groups[] = array('ad_ids' => $ad_group_ids_array, 'restrictions' => $r, 'group_id' => $group_id);  
		}

		$place_ids_query = [
			'post_type'      => $place_slug,
			'posts_per_page' => -1,
			'fields'         => 'ids' 
		];
		$place_ids = get_posts( $place_ids_query ); 
		$places = array();
		foreach($place_ids as $place_id){
			$forms       = get_post_meta( $place_id, 'forms', true );
			$places[] = array('forms' => $forms, 'place_id' => $place_id);  
		}

		$uam_settings = $this->settings_mgr->general_settings; 

		$table_name_events = $cc_uam_config['table_name_events'];
		$res = $wpdb->get_results("SELECT * FROM $table_name_events ORDER BY id DESC LIMIT 1", ARRAY_A);
		$max_event_id = is_array($res) && count($res) > 0 ? intval($res[0]['id']) : null; 

		$data = array(
			'php_version' => $phpversion,
			'wp_version' => $wp_version,
			'plugins' => $plugins,
			'active_plugins' => $active_plugins,
			'theme' => $theme_data,
			'locale' => $locale,
			'plugin_id' => $plugin_id,
			'adverts' => $adverts,
			'groups' => $groups,
			'places' => $places,
			'uam_settings' => $uam_settings,
			'max_event_id' => $max_event_id,

		);

		$site_url = \get_site_url();
		$res = array(
			'site_url' => $site_url,
			'type' => 'uam_info',
			'context' => 'uam',
			'data' => json_encode($data),
			'version' => 1 
		);

		$args = array(
          'body' => $res, 
          'blocking' => false,
          'sslverify' => false
        );
		\wp_remote_post($cc_uam_config['wpps_url'].'/event/2.0/', $args);
		wp_send_json($res);
	}

	public function pre_update_settings($new_value, $old_value ){ 
		\set_transient('codeneric/uam/data_changed', true, 60); 
		return $new_value; 
	}

	public function send_report(){
		$start = time() - WEEK_IN_SECONDS;
		$end = time(); 
		$ad_ids = $this->stats_calc->get_ad_ids_with_events_in_time_period($start, $end);
		$res = array();
		foreach($ad_ids as $id){
			$query = array(
				'ad_id' => $id,
				'type' =>  'view' ,
				'metric' =>  'total',
				'place_id' => null,
				'ranges' => array(array($start, $end))
			);

			$query_res = $this->stats_calc->query($query); 
			$total_views = count($query_res) > 0 ? $query_res[0]->data : 0;   
			
			$res[] = array(
				'total_views' => $total_views,
				'ad_id' => $id,
				'ad_title' => get_the_title($id)
			);

		}
		$email = $this->generate_report_email($res, $start, $end); 

		$admin_mail = get_option('admin_email'); 
		$headers = array(
			'Content-Type: text/html; charset=UTF-8',
			'Reply-To: Team Codeneric <support@codeneric.com>'
			); 
		$r = wp_mail($admin_mail, $email['subject'], $email['body'], $headers); 
		if(wp_doing_ajax()){ 
			wp_send_json($res);
		}
	}

	public function generate_report_email($data, $start, $end){
		global $cc_uam_config;
		$body = \file_get_contents(__DIR__.'/../assets/report.html');
		$table = "<table>%s</table>";
		$table = sprintf($table, "
			<tr>
				<th>Advert ID</th>
		 		<th>Advert title</th>
		 		<th>Total views</th>
			</tr>
			%s
			"); 
		$sum = 0;
		foreach($data as $d){
			$ad_id = $d['ad_id'];
			$ad_title = $d['ad_title'];
			$total_views = $d['total_views'];
			$table = sprintf($table, "
			<tr>
				<td>$ad_id</td>
		 		<td>$ad_title</td>
		 		<td>$total_views</td>
			</tr>
			%s
			");
			$sum += $total_views;
		}

		$threshold = $cc_uam_config['weekly_traffic_warning_threshold']; 

		$status_is_good = $cc_uam_config['has_premium_ext'] || $sum < $threshold; //is able to handle current traffic?

		$status_is_good = $cc_uam_config['env'] === 'development' ? false : $status_is_good; 
		
		$status_text = $status_is_good ? 'good' : 'high traffic';
		$subject = "UAM Advert Report | Status: $status_text";  

		$table = sprintf($table, "
			<tr>
				<td></td>
		 		<td></td>
		 		<td>Sum: $sum</td>
			</tr>
			%s
			");

		$table = sprintf($table, ''); 
		
		$body = str_replace('%WEBSITE-NAME%', get_home_url(), $body);
		$body = str_replace('%START-DATE%', date('m/d/Y', $start), $body);
		$body = str_replace('%END-DATE%', date('m/d/Y', $end), $body); 
		$body = str_replace('%TABLE%', $table, $body); 

		if($status_is_good){ 
			$body = str_replace('%ACTION%', '<p><strong>Status:</strong> <span style="color:green;">good</span></p>', $body); 
		}else{

			$link = "https://codeneric.com/cart/?add-to-cart=17&variation_id=124&attribute_billing-period=Yearly&utm_source=uam-base-plugin&utm_medium=email&utm_campaign=recommend-premium";
			$atag = "<a href=\"$link\">Ultimate Ads Manager Premium</a>";
			$body = str_replace('%ACTION%', '<p><strong>Status:</strong> <span style="color:orange;"><strong>high traffic</strong></span> </p>
			<p>
			Your site serves to many users, which is awesome! 
			But we strongly recommend to upgrade to '.$atag.' to handle all the views and clicks produced by the traffic reliably and keep your adverts working properly.</p>
			', $body);  

		}

		return array('body' => $body, 'subject' => $subject); 
	}

	public function schedule_reports(){
		global $cc_uam_config;
		$name = $cc_uam_config['weekly_report_schedule_name'];
		if ( ! wp_next_scheduled( $name ) ) {
			$should_schedule = defined ( 'CODENERIC_UAM_WEEKLY_REPORT_ENABLED' ) 
				? constant('CODENERIC_UAM_WEEKLY_REPORT_ENABLED') : true; 
			
			$next_run = strtotime('next monday');
			if($should_schedule){ //send reports to free users only
				wp_schedule_single_event( $next_run, $name );
			}
		}
	}



}



