<?php
/**
* Elementor oEmbed Widget.
 *
 * Elementor widget that inserts an embbedable content into the page, from any given URL.
 *
 * @since 1.0.0
 */
class Elementor_Codeneric_Widget extends \Elementor\Widget_Base {

	/**
	 * Get widget name.
	 *
	 * Retrieve oEmbed widget name.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'codeneric_uam_place';
	}

	/**
	 * Get widget title.
	 *
	 * Retrieve oEmbed widget title.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'Ultimate Ads Manager: Place', 'ultimate-ads-manager' );
	}

	/**
	 * Get widget icon.
	 *
	 * Retrieve oEmbed widget icon.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'fa fa-ad';
	}

	/**
	 * Get widget categories.
	 *
	 * Retrieve the list of categories the oEmbed widget belongs to.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'codeneric-uam' ];
	}

	/**
	 * Register oEmbed widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 * @access protected
	 */


    public function generateRandomString($length = 16) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
	protected function _register_controls() {
       
        $this->start_controls_section(
			'layout_section',
			[
				'label' => __( 'General', 'ultimate-ads-manager' ),
			]
        );
        
        $this->add_control(
			'title',
			[
				'label' => __( 'Place name', 'ultimate-ads-manager' ),
				'type' => \Elementor\Controls_Manager::TEXT,
                'placeholder' => __( 'e.g. header', 'ultimate-ads-manager' ),
                
			]
		);
		$this->add_control(
			'hr',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);
        $this->add_control(
			'width',
			[
				'label' => __( 'Width', 'ultimate-ads-manager' ),
				'default' => 100,
				'type' => \Elementor\Controls_Manager::NUMBER				
			]
		);
		$this->add_control(
			'width_unit',
			[
				'label' => __( 'Width Unit', 'plugin-domain' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => '%',
				'options' => [
					'px'  => __( 'Pixel', 'plugin-domain' ),
					'%' => __( 'Percent [%]', 'plugin-domain' )				
				],
			]
		);


		$this->add_control(
			'ratio',
			[
				'label' => __( 'Ratio', 'ultimate-ads-manager' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'placeholder' => __( 'e.g. 4/3 or 1.333', 'ultimate-ads-manager' ),
				'default' => "8/1",
                'description' => __( "Desired place ratio. [width/height]", 'ultimate-ads-manager' )      
			]
		);

		$this->add_control(
			'version',
			[
				'type' => \Elementor\Controls_Manager::HIDDEN,
				'default' => '1'				
			]
		);
	
       
		
		$this->end_controls_section();

	}


	public function parse_ratio($ratio) {
		$ret = 1.;

		if(is_numeric($ratio))
			$ret = (float) $ratio;


		$a = explode ("/", $ratio);  

		if(count($a) === 2) {
			$ret = (float) $a[0] / (float) $a[1];
		}


		if($ret > 0)
			return $ret;
		else return 1.;

	}

	
	protected function render() {
		$settings = $this->get_settings_for_display();       
	
		$ratio =  $this->parse_ratio($settings["ratio"]); 	
		$w = $settings["width"];
		$unit = $settings["width_unit"];
	
        if ( \Elementor\Plugin::$instance->preview->is_preview_mode() ||  \Elementor\Plugin::$instance->editor->is_edit_mode() ) {
            // do stuff for edit mode
			
			$style = "background:#338DC9;position:relative;box-sizing:border-box;vertical-align:bottom;outline: 4px dashed rgba(0,0,0,0.5);outline-offset: -4px;";
			
			
			if( $settings["width_unit"] === "px") {
								
				$h = $w  *  (1/$ratio);			
				$style .= "width:${w}px;height:${h}px;";
			}
			else {				
				$p = 100 * (1 / $ratio) * ($w / 100);				

				$style .= "width:${w}%;padding-top:${p}%";
			}
			echo "<style> #codeneric_uam_elementor_place_widget_dummy:after { content: 'Ad'; position:absolute; top:25%; width:100%; text-align:center;color:white;font-weight:bold;font-size:12px; } </style>";
            echo "<div id='codeneric_uam_elementor_place_widget_dummy'  style='$style' />";
       } else {
           // not edit mode     
			
			
			echo "[uam_place unit=\"$unit\" width=\"$w\" ratio=\"$ratio\"]";
		
			//    $width = $settings["width"];
        	//    $height = $settings["height"];
       		//    echo "<div  style='width:$width;height:$height;background:url(https://codeneric.com/wp-content/uploads/2018/03/uam-megaphone-illustration-1024x576.png);background-size:cover;' />";

       }



	}

}
